//package mobicomp.layer;
package server;

/**
 * Represents a thread that can be reused. Once it has finished its
 * task it will put itself back into the thread pool, awaiting reuse.
 */
public class ReusableThread implements Runnable {
	/** The thread pool we belong to. */
	private ThreadPool pool;

	/** The task that we do. */
	private Task task;
	/** The arguments for the task. */
	private Object[] arguments;

	/** Signify whether we're running or not. */
	private boolean idle = true;
	/** whether we should exit */
	private boolean cont = true;

	ReusableThread(Task task, ThreadPool pool) {
		this.task = task;
		this.pool = pool;
	}

	void setThreadPool(ThreadPool pool) {
		this.pool = pool;
	}

	/**
	 * Indicate that this thread can now be used.
	 */
	synchronized public void setReady(Object[] args) {
		idle = false;
		arguments = args;
		notify();
	}

	synchronized public void run() {
		while (cont) {
			/* wait to be used */
			if (idle) {
				try {
					wait();
				} catch (InterruptedException ie) {
					continue;
				}
			}

			/* now we're ready to do the task */
			task.doTask(arguments);

			/* put ourselves pack in the pool */
			idle = true;
			cont = pool.putBack(this);
		}
	} //run()

}
